/* File save_log.c
 * January 12, 2007
 * by Doug Mink */

/* Read log file and put remarks into REMARK header parameters of images */

/* Based on tex_fast by Doug Mink, 18 May 2001 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <dirent.h>
#include "fitsfile.h"

#define MAXC 640		/* Length of input line */
#define MAXW 640		/* Maximum word length	*/

static char *dirmatch();

int
main (argc, argv)

int argc;
char **argv;

{
	int	i, ntok, naxis;
	int	added;

	FILE	*log;
	struct Tokens tokens;	/* Token structure returned */
	struct Tokens tokenl;	/* Token structure returned */
	char *irafheader;	/* IRAF image header */
	int iraffile;		/* 1 if IRAF image, 0 if not */
	int fitsfile;		/* 1 if FITS image, 0 if not */
	char *header;		/* FITS image header */
	char *image;
	char *instrument;
	int lhead;		/* Maximum number of bytes in FITS header */
	int nbhead;		/* Actual number of bytes in FITS header */
	int verbose;
	int lroot;
	int iarg;
	int nr = 0;
	int imageread;
	char *logroot;
	char *newfile;
	char	s[MAXC],
		temp[MAXW],
		dirname[MAXC],
		logfile[MAXC],
	        imroot[MAXC],
	        imroot1[MAXC],
	        imfile[MAXC],
	        comment[MAXC];

	/* Logfile root name is first command line argument */
	if (--argc <= 0) {
	    fprintf (stderr, "SAVE_LOG: Add comments from log file to image headers\n");
	    fprintf (stderr, "usage: save_log [-v] instrument logroot (.lis is added)\n");
	    fprintf (stderr, "       -v prints what is being done\n");
	    fprintf (stderr, "       instrument is 1sh 2sh echelle fast\n");
	    fprintf (stderr, "       .lis is added to logroot\n");
	    exit (-1);
	    }
	if (argv[1][0] == '-' && argv[1][1] == 'v' && --argc >= 0) {
	    verbose = 1;
	    iarg = 2;
	    }
	else {
	    verbose = 0;
	    iarg = 1;
	    }
	strcpy (dirname, ".");

	/* Instrument is first command line argument */
	if (--argc >= 0)
	    instrument = argv[iarg];
	else {
	    fprintf (stderr, "SAVE_LOG: No instrument\n");
	    exit (-1);
	    }

	/* Log file root is second command line argument */
	if (--argc >= 0)
	    logroot = argv[iarg+1];
	else {
	    fprintf (stderr, "SAVE_LOG: No log file\n");
	    exit (-1);
	    }

	/* Find log input file */
	strcpy (logfile, logroot);
	strcat (logfile, ".lis");
	if (verbose)
	    fprintf (stderr,"SAVE_LOG: Logfile root is %s\n", logfile);
 
	if ((log = fopen (logfile,"r")) == NULL) { 
	    fprintf (stderr, "Error opening log file %s\n", logfile); 
	    exit (-1); 
	    } 
 
	/* Get lines until end of log file is reached */
	for (i = 0; i < MAXC; i++)
	    s[i] = (char) 0;
	while (fgets(s, MAXC, log) != NULL) {
	    nr++;

	    /* Parse line from input file */
	    setoken (&tokens, s, "");

	    /* Get last token on line, which should be the comment */
	    ntok = tokens.ntok;
	    (void) getoken (&tokens, ntok, comment, MAXC);
	    if (strlen (comment) < 2)
		continue;

	    /* Find file name from first entry in this line of log */
	    fitsfile = 0;
	    iraffile = 0;
	    (void) getoken (&tokens, 1, imroot, MAXC);
	    lroot = strlen (imroot);
	    i = 0;

	    /* Remove backslashes */
	    while (i < lroot) {
		if (imroot[i] == '\\') {
		    strcpy (imroot+i, imroot+i+1);
		    lroot--;
		    }
		else
		    i++;
		}
	    strcpy (imfile, imroot);
	    strcat (imfile, ".imh");
	    if (!access (imfile, F_OK))
		iraffile = 1;
	    else {
		strcpy (imfile, imroot);
		strcat (imfile, ".fits");
		if (!access (imfile, F_OK))
		    fitsfile = 1;
		}

	    /* If not found, try wildcarding end of file name */
	    if (!fitsfile && !iraffile) {
		newfile = dirmatch (dirname, imroot);
		if (newfile == NULL) {
		    strcpy (imfile, imroot);
		    strcat (imfile, ".imh");
		    newfile = dirmatch (dirname, imfile);
		    }
		else {
		    strcpy (imfile, newfile);
		    free (newfile);
		    if (isfits (imfile))
			fitsfile = 1;
		    if (isiraf (imfile))
			iraffile = 1;
		    }
		}

	    /* If not found, try first of compressed entry */
	    if (!fitsfile && !iraffile) {
		setoken (&tokenl, imroot, ".-");
		ntok = tokenl.ntok;
		if (ntok > 4) {
		    (void) getoken (&tokenl, 1, temp, MAXW);
		    i = atoi (temp);
		    sprintf (imroot1, "%04d", i);
		    strcat (imroot1, ".");
		    (void) getoken (&tokenl, -5, temp, MAXW);
		    strcat (imroot1, temp);
		    strcpy (imfile, imroot1);
		    strcat (imfile, ".imh");
		    if (!access (imfile, F_OK))
			iraffile = 1;
		    else {
			strcpy (imfile, imroot1);
			strcat (imfile, ".fits");
			if (!access (imfile, F_OK))
			    fitsfile = 1;
			else
			    fprintf (stderr,"Cannot find image file %s.\n",
				     imfile);
			}
		    }
		}

	    /* If still not found, skip this line and go to next */
	    if (!fitsfile && !iraffile) {
		fprintf (stderr, "Cannot find image file %s.[imh or fits]\n",
			 imroot);
		continue;
		}

	    /* Read image header from IRAF .imh or FITS file */
	    if (iraffile) {
		if (!strncmp (instrument, "4sh", 3))
		    strcat (imfile, ",1");
		if ((irafheader = irafrhead (imfile, &lhead)) != NULL) {
		    iraffile = 1;
		    if ((header=iraf2fits(imfile,irafheader,lhead,&nbhead))==NULL) {
			fprintf (stderr, "Cannot translate IRAF header %s/n",imfile);
			free (irafheader);
			exit (1);
			}
		    }
		}
	    else if (fitsfile) {
		if (access (imfile, F_OK)) {
		    fprintf (stderr, "Cannot find image file %s\n", imfile);
		    continue;
		    }
		if (!strncmp (instrument, "4sh", 3))
		    strcat (imfile, ",1");
		if ((header = fitsrhead (imfile, &lhead, &nbhead)) != NULL) {
		    hgeti4 (header,"NAXIS",&naxis);
		    if (naxis > 0) {
			if ((image = fitsrimage (imfile, nbhead, header)) == NULL) {
			    fprintf (stderr, "No FITS image in %s\n", imfile);
			    imageread = 0;
			    }
			else
			    imageread = 1;
			}
		    else
			fprintf (stderr,"Rewriting primary header, copying rest\n");
		    }
		else {
		    fprintf (stderr, "Cannot read image file %s\n", imfile);
		    continue;
		    }
		}

	    /* Add remark to file header if one is not already there */
	    if (!ksearch (header, "REMARK"))
		hputs (header, "REMARK", comment);
	    else {
		printf ("%s already has REMARK.\n", imfile);
		continue;
		}

	    /* Write fixed header to output file */
	    added = 0;
	    if (iraffile) {
		if (irafwhead (imfile,lhead,irafheader,header) <= 0)
		    printf ("%s could not be written.\n", imfile);
		else
		    added = 1;
		free (irafheader);
		}
	    else if (imageread) {
		if (fitswimage (imfile, header, image) <= 0)
		    printf ("%s could not be written.\n", imfile);
		else
		    added = 1;
		free (image);
		}
	    else
		printf ("%s could not be written.\n", imfile);

	    if (added && verbose)
		printf ("%s: %s\n", imfile, comment);
	    for (i = 0; i < MAXC; i++)
		s[i] = (char) 0;
	    }

	(void) fclose (log);
	if (nr < 1)
	    fprintf (stderr, "SAVELOG: No data or blank first line in %s.\n",
		     logfile);
	exit (0);
}


/* DIRMATCH -- Match pattern in directory, returning file name or NULL */

static char *
dirmatch (dirname, template)

char	*dirname;	/* Directory path name */
char	*template;	/* Template string */

{
DIR *dirp;
struct dirent *dp;
char *filename;
int nt, lfile;

        nt = strlen (template);

	filename = NULL;
        dirp = NULL;
        dirp = opendir (dirname);
        if (dirp == NULL) {
            printf ("DIRTEMP:  Directory '%s' not found\n",dirname);
            return (filename);
            }
        for (dp = readdir (dirp); dp != NULL; dp = readdir (dirp))
            if (!strncmp (dp->d_name, template, nt)) {
		lfile = strlen (dp->d_name);
		filename = (char *) calloc (lfile, 1);
                strcpy (filename,dp->d_name);
                break;
                }
        closedir (dirp);
        return (filename);
}
/* May 24 2002	New program
 *
 * Feb 14 2002	Add maximum length to getoken() calls
 *
 * Jun 30 2003	Print error message if no lines read
 *
 * Nov 17 2005	Clean up code using lint
 *
 * Jan 12 2007	Pass pointer to token string structure to getoken()
 */
