/* tex_log.c
 * Latest update by Doug Mink, January 11, 2007
 * Originally by Tod R. Lauer, April 24, 1995
 */

/* This is the main program for generating TeX output from lines
   saved by the logging routines.  In this main program, we simply
   look for the main logging rootname and the instrument type.
   Once the instrument type is known, the program calls the 
   instrument-specific formatting routine. */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "obslog.h"

#define MAXC 640		/* Length of input line */

int
main (ac, argv)

int ac;
char **argv;

{
	char	instrument[MAXC],
		libpath[MAXC],
		logfile[MAXC];

	int	npage, iarg, verbose, tex_ccd(), tex_ech(), tex_fast();

	/* Logfile root name is first command line argument */
	iarg = 1;
	if (--ac > 0) {
	    if (!strncmp (argv[iarg], "-v", 2)) {
		verbose = 1;
		iarg++;
		}
	    else
		verbose = 0;
            strcpy (logfile, argv[iarg++]);
	    }
	else {
	    fprintf (stderr, "TEX_LOG: Write TeX observing log from ASCII file\n");
	    fprintf (stderr, "usage:   tex_log logroot instrument library_dir\n");
	    fprintf (stderr, "         logroot (.lis is added) (usually obslog)\n");
	    fprintf (stderr, "         instrument is fast, echelle, 1sh, 4sh\n");
	    fprintf (stderr, "         library_dir is obslog$lib\n");
	    printf ("0\n");
	    exit (-1);
	    }
	/* fprintf (stderr,"TEX_LOG: Logfile root is %s\n", logfile); */

	/* Instrument name is second command line argument */
	if (--ac > 0)
            strcpy (instrument, argv[iarg++]);
	else {
	    printf ("0\n");
	    exit (-1);
	    }
	/* fprintf (stderr,"TEX_LOG: Instrument is %s\n", instrument); */

	/* Library pathname is third command line argument */
	if (--ac > 0)
            strcpy (libpath, argv[iarg]);
	else {
	    printf ("0\n");
	    exit (-1);
	    }
	/* fprintf (stderr,"TEX_LOG: obs_logging library is %s\n", libpath); */

	if (verbose)
	    fprintf (stderr, "tex_log %s %s %s\n", logfile, instrument, libpath);

	/* Call appropriate log translation subroutine for instrument */
	if (strcmp(instrument,"1sh") == 0)
	    npage = tex_ccd (logfile, libpath);
	else if (strcmp(instrument,"4sh") == 0)
	    npage = tex_ccd (logfile, libpath);
	else if (strncmp(instrument,"hsp",3) == 0 ||
		 strncmp(instrument,"hectos",6) == 0 )
	    npage = tex_hspec (logfile, libpath, verbose);
	else if (strncmp(instrument,"hec",3) == 0 ||
		 strncmp(instrument,"hectoc",6) == 0 )
	    npage = tex_hchelle (logfile, libpath, verbose);
	else if (strcmp(instrument,"ech") == 0)
	    npage = tex_ech (logfile, libpath);
	else if (strcmp(instrument,"fast") == 0) {
	    if (verbose)
		fprintf (stderr,"TEX_LOG: Converting %s.lis to %s.tex\n",
			 logfile, logfile);
	    npage = tex_fast (logfile, libpath, verbose);
	    }
	else {
	    fprintf (stderr, "Unknown instrument: %s\n", instrument);
	    printf ("0\n");
	    exit (-1);
	    }
	printf ("%d\n", npage);
	exit (0);
} 
/* May 30 2001	Rewritten by Doug Mink
 *
 * Mar  4 2004	Add Hectospec and Hectochelle
 *
 * Jan 11 2007	Include string.h and stdlib.h
 */
